/*
 * SchedulerDAOImpl.java
 */

package scheduler.dao;

import java.io.*;
import java.sql.Date;
import java.sql.Time;
import java.text.*;
import java.util.*;
import scheduler.domain.*;
import scheduler.util.*;

public class SchedulerDAOImpl implements SchedulerDAO
{
	static final int FIELD_KIND         = 0;
	static final int FIELD_TIME         = 1;
	static final int FIELD_MEMO         = 2;
	static final int FIELD_DATE         = 3;
	static final int FIELD_OPENING      = 3;
	static final int FIELD_CLOSING      = 4;
	static final int FIELD_DAY_OF_WEEK  = 5;
	static final int FIELD_DAY_OF_MONTH = 5;

	static final int SIZE_FIXED         = 4;
	static final int SIZE_WEEKLY        = 6;
	static final int SIZE_MONTHLY       = 6;

	static final String KIND_FIXED   = "F";
	static final String KIND_WEEKLY  = "W";
	static final String KIND_MONTHLY = "M";

	private String location;
	private String encoding;

	public void init() throws Exception {
		location = AppUtils.getProperty("data.location", "scheduler.dat");
		encoding = AppUtils.getProperty("data.encoding");
	}

	public void load(Scheduler scheduler) throws Exception {
		BufferedReader in = null;
		try {
			in = new BufferedReader(openReader());
			loadSchedules(scheduler, in);
		} finally {
			if (in != null) in.close();
		}
	}

	protected Reader openReader() throws IOException {
		if (encoding == null || encoding.length() == 0) {
			return new FileReader(location);
		} else {
			return new InputStreamReader(
					new FileInputStream(location), encoding);
		}
	}

	protected void loadSchedules(Scheduler scheduler, BufferedReader in)
			throws IOException {
		String line = in.readLine();
		while (line != null) {
			loadSchedule(scheduler, toList(line));
			line = in.readLine();
		}
	}

	protected void loadSchedule(Scheduler scheduler, List values) {
		String kind = (String)values.get(FIELD_KIND);
		if (kind.equals(KIND_FIXED)) {
			scheduler.addFixedSchedule(toFixedSchedule(values));
		} else if (kind.equals(KIND_WEEKLY)) {
			scheduler.addWeeklySchedule(toWeeklySchedule(values));
		} else {
			throw new RuntimeException("kind=" + kind);
		}
	}

	protected List toList(String line) {
		List list = new ArrayList();
		int start = 0;
		int index = line.indexOf(',');
		while (index >= 0) {
			list.add(line.substring(start, index).trim());
			start = index + 1;
			index = line.indexOf(',', start);
		}
		list.add(line.substring(start).trim());
		return list;
	}

	protected FixedSchedule toFixedSchedule(List values) {
		Date date = Date.valueOf((String)values.get(FIELD_DATE));
		Time time = Time.valueOf((String)values.get(FIELD_TIME));
		String memo = (String)values.get(FIELD_MEMO);
		return new FixedSchedule(date, time, memo);
	}

	protected WeeklySchedule toWeeklySchedule(List values) {
		int dayOfWeek = Integer.parseInt(
				(String)values.get(FIELD_DAY_OF_WEEK));
		Time time = Time.valueOf((String)values.get(FIELD_TIME));
		String memo    = (String)values.get(FIELD_MEMO);
		String opening = (String)values.get(FIELD_OPENING);
		String closing = (String)values.get(FIELD_CLOSING);
		WeeklySchedule schedule = new WeeklySchedule(dayOfWeek, time, memo);
		if (opening != null && opening.length() > 0) {
			schedule.setOpening(Date.valueOf(opening));
		}
		if (closing != null && closing.length() > 0) {
			schedule.setClosing(Date.valueOf(closing));
		}
		return schedule;
	}

	public void save(Scheduler scheduler) throws Exception {
		PrintWriter out = null;
		try {
			out = new PrintWriter(new BufferedWriter(openWriter()));
			saveFixedSchedules(out, scheduler.getFixedSchedules());
			saveWeeklySchedules(out, scheduler.getWeeklySchedules());
			out.flush();
		} finally {
			if (out != null) out.close();
		}
	}

	protected Writer openWriter() throws IOException {
		if (encoding == null || encoding.length() == 0) {
			return new FileWriter(location);
		} else {
			return new OutputStreamWriter(
					new FileOutputStream(location), encoding);
		}
	}

	protected void saveFixedSchedules(PrintWriter out, SortedMap schedules) {
		for (Iterator i = schedules.values().iterator(); i.hasNext(); ) {
			ScheduleList list = (ScheduleList)i.next();
			for (Iterator j = list.iterator(); j.hasNext(); ) {
				FixedSchedule schedule = (FixedSchedule)j.next();
				List values = toList(schedule);
				out.println(toLine(values));
			}
		}
	}

	protected void saveWeeklySchedules(
			PrintWriter out, ScheduleList[] schedules) {
		for (int i = 0; i < schedules.length; ++i) {
			ScheduleList list = schedules[i];
			for (Iterator j = list.iterator(); j.hasNext(); ) {
				WeeklySchedule schedule = (WeeklySchedule)j.next();
				List values = toList(schedule);
				out.println(toLine(values));
			}
		}
	}

	protected String toLine(List values) {
		StringBuffer buf = new StringBuffer();
		Iterator i = values.iterator();
		if (i.hasNext()) {
			buf.append(i.next());
			while (i.hasNext()) {
				buf.append(", ");
				buf.append(i.next());
			}
		}
		return buf.toString();
	}

	protected List toList(FixedSchedule schedule) {
		List values = Arrays.asList(new String[SIZE_FIXED]);
		values.set(FIELD_KIND, KIND_FIXED);
		values.set(FIELD_DATE, schedule.getDate().toString());
		values.set(FIELD_TIME, schedule.getTime().toString());
		values.set(FIELD_MEMO, schedule.getContent().getMemo());
		return values;
	}

	protected List toList(WeeklySchedule schedule) {
		List values = Arrays.asList(new String[SIZE_WEEKLY]);
		values.set(FIELD_KIND, KIND_WEEKLY);
		values.set(FIELD_DAY_OF_WEEK,
				Integer.toString(schedule.getDayOfWeek()));
		values.set(FIELD_TIME, schedule.getTime().toString());
		values.set(FIELD_MEMO, schedule.getContent().getMemo());
		Date opening = schedule.getOpening();
		Date closing = schedule.getClosing();
		values.set(FIELD_OPENING, (opening == null) ? "" : opening.toString());
		values.set(FIELD_CLOSING, (closing == null) ? "" : closing.toString());
		return values;
	}
}
