/*
 * Scheduler.java
 */

package scheduler.domain;

import java.sql.Date;
import java.util.*;
import scheduler.util.*;

public class Scheduler
{
	static final int DAYS_OF_WEEK = 7;

	// SortedMap<Date, ScheduleList>
	private SortedMap      fixedSchedules  = new TreeMap();
	private ScheduleList[] weeklySchedules = new ScheduleList[DAYS_OF_WEEK];
	private SchedulerDAO dao = null;

	public Scheduler() {
		for (int i = 0; i < DAYS_OF_WEEK; ++i) {
			weeklySchedules[i] = new ScheduleList();
		}
	}

	public SortedMap getFixedSchedules() {
		return fixedSchedules;
	}

	public SortedMap getFixedSchedules(Date from, Date to) {
		if (from == null) return fixedSchedules.headMap(to);
		if (to   == null) return fixedSchedules.tailMap(from);
		return fixedSchedules.subMap(from, to);
	}

	public SortedMap getFixedSchedules(int year) {
		Date from = DateTimeUtils.toSQLDate(year, 1, 1);
		Date to   = DateTimeUtils.toSQLDate(year + 1, 1, 1);
		return getFixedSchedules(from, to);
	}

	public SortedMap getFixedSchedules(int year, int month) {
		Date from = DateTimeUtils.toSQLDate(year, month, 1);
		Date to   = DateTimeUtils.toSQLDate(year, month + 1, 1);
		return getFixedSchedules(from, to);
	}

	public ScheduleList getFixedSchedules(Date date) {
		ScheduleList list = (ScheduleList)fixedSchedules.get(date);
		if (list == null) {
			list = new ScheduleList();
			fixedSchedules.put(date.clone(), list);
		}
		return list;
	}

	public ScheduleList getFixedSchedules(int year, int month, int day) {
		return getFixedSchedules(DateTimeUtils.toSQLDate(year, month, day));
	}

	public void addFixedSchedule(FixedSchedule schedule) {
		Date date = schedule.getDate();
		getFixedSchedules(date).add(schedule);
	}

	public void removeFixedSchedule(FixedSchedule schedule) {
		Date date = schedule.getDate();
		getFixedSchedules(date).remove(schedule);
	}

	public void clearFixedSchedules() {
		for (Iterator i = fixedSchedules.values().iterator(); i.hasNext(); ) {
			ScheduleList list = (ScheduleList)i.next();
			list.clear();
		}
		fixedSchedules.clear();
	}

	public ScheduleList[] getWeeklySchedules() {
		return weeklySchedules;
	}

	public ScheduleList getWeeklySchedules(int dayOfWeek) {
		return weeklySchedules[dayOfWeek - Calendar.SUNDAY];
	}

	public void addWeeklySchedule(WeeklySchedule schedule) {
		int dayOfWeek = schedule.getDayOfWeek();
		getWeeklySchedules(dayOfWeek).add(schedule);
	}

	public void removeWeeklySchedule(WeeklySchedule schedule) {
		int dayOfWeek = schedule.getDayOfWeek();
		getWeeklySchedules(dayOfWeek).remove(schedule);
	}

	public void clearWeeklySchedules() {
		for (int i = 0; i < DAYS_OF_WEEK; ++i) {
			weeklySchedules[i].clear();
		}
	}

	public void clearAll() {
		clearFixedSchedules();
		clearWeeklySchedules();
	}

	public void load() throws Exception {
		clearAll();
		getSchedulerDAO().load(this);
	}

	public void save() throws Exception {
		getSchedulerDAO().save(this);
	}

	protected SchedulerDAO getSchedulerDAO() throws Exception {
		if (dao != null) return dao;
		synchronized (this) {
			if (dao != null) return dao;
			dao = createSchedulerDAO();
			return dao;
		}
	}

	protected SchedulerDAO createSchedulerDAO() throws Exception {
		String name = AppUtils.getProperty(
				"scheduler.dao", "scheduler.dao.SchedulerDAOImpl");
		SchedulerDAO dao = (SchedulerDAO)Class.forName(name).newInstance();
		dao.init();
		return dao;
	}

	static final String LINE_SEPARATOR = System.getProperty("line.separator");

	public String toString() {
		List list = new ArrayList();
		for (Iterator i = fixedSchedules.values().iterator(); i.hasNext(); ) {
			ScheduleList sl = (ScheduleList)i.next();
			list.addAll(sl.getAll());
		}
		for (int i = 0; i < DAYS_OF_WEEK; ++i) {
			list.addAll(weeklySchedules[i].getAll());
		}
		StringBuffer buf = new StringBuffer();
		Iterator i = list.iterator();
		if (i.hasNext()) {
			buf.append(i.next());
			while (i.hasNext()) {
				buf.append(LINE_SEPARATOR).append(i.next());
			}
		}
		return buf.toString();
	}
}
